==> Cosmos8.02.tar.gz: Cosmos8.02/Tracking/cinteNEP.f <==
!            treat interaction of MovedTrack for non e,g
!
      subroutine cinteNEP
      implicit none

#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
!
!
!          used to judge if user hook should be called 
!          after MovedTrack interacted.
!
!
      character*80 msg
!
!   **  ptcl stacking is done in each subroutine; should be changed
!           (except for hadronic interactions)
!////////////
!      logical deb
!      common /cccdeb/deb
!/////////////

      
!/////////
!      if(deb) then
!         write(*,*) 'NEP:proc=',  IntInfArray(ProcessNo).process
!      endif
!////////////

      if(IntInfArray(ProcessNo)%process .eq. 'knock') then
         call csetKnock
      else
         if(MovedTrack%p%code .eq. kpion) then
            call cintePion
         elseif(MovedTrack%p%code .eq. kkaon) then
            call cinteKaon
         elseif(MovedTrack%p%code .eq. knuc) then
            call cinteNuc
         elseif(MovedTrack%p%code .eq. kmuon) then
            call cinteMuon
         elseif( MovedTrack%p%code .eq. kgnuc ) then
            call cinteHeavy
         elseif(MovedTrack%p%code .ge. kalfa .and.
     *           MovedTrack%p%code .le. khvymax  ) then
!cc            elseif(MovedTrack.p.code .ge. kdeut .and.
            call cinteHeavy
         elseif(MovedTrack%p%code .eq. ktriton) then
            call cinteHeavy
         elseif(MovedTrack%p%code .eq. kdmes ) then
            call cinteDmes
         elseif(MovedTrack%p%code .eq. knnb) then
            call cintennb
         elseif(MovedTrack%p%code .eq. kddb) then
            call cinteddb
         elseif(MovedTrack%p%code .eq. keta) then
            call cinteEta
         elseif(MovedTrack%p%code .eq. kgzai) then
            call cinteGzai
         elseif(MovedTrack%p%code .eq. kbomega) then
            call cinteBomega
         elseif(MovedTrack%p%code .eq. klambda) then
            call cinteLambda
         elseif(MovedTrack%p%code .eq. klambdac) then
            call cinteLambdac
         elseif(MovedTrack%p%code .eq. ksigma) then
            call cinteSigma
         elseif(MovedTrack%p%code .eq. krho) then
            call cinterho
         elseif(MovedTrack%p%code .eq. komega) then
            call cinteomega
         elseif(MovedTrack%p%code .eq. kphi) then
            call cintephi
         else
            write(0,*) ' E=', MovedTrack%p%fm%p(4)
            write(msg, *) ' ptcl =', MovedTrack%p%code,
     *        ' interaction=', 
     *         IntInfArray(ProcessNo)%process,
     *        ' should not occure'
            call cerrorMsg(msg, 0)
         endif
      endif
      end     
      subroutine csetKnock
      implicit none
#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
!

       real*8 e1, er, tmp, cos1, cosr,  cs, sn, sinr, sine
       type(coord)::dc
       type(coord)::dc1
       type(coord)::dcr
!
       type(track)::aTrack

       integer idx   ! 
 
       aTrack = MovedTrack
!                                surv  elec  surv   elec
       call  cKnockea(aTrack%p,  e1,   er,  cos1,   cosr)
!        we must put recoil electron into Pwork first and
!        then survival high energy ptcl.  This order will
!        be reversed when particles in Pwork are stacked.
!      (this inversion is to manage nuclear interactions where
!       the leading particle is put last in Pwork).
!        Therefore the low energy electron is first extracted.
!  Otherwise, stack overflow may take place for inclinde muons
!     (<= v5.33)
!         idx is to resolve the problem with a minumum effort.
!
!          survival paticle angle negligible always so
!          you may  put dc.r = (0, 0, 1)
       tmp=1.d0-cos1*cos1
       if(tmp .lt. 0.d0) then
          tmp=0.d0
          cos1=1.d0
       endif
       sine=sqrt(tmp)

       call kcossn(cs, sn)
       dc%r(1) = cs*sine
       dc%r(2) = sn*sine
       dc%r(3) = cos1
!           convert angle
       call ctransVectZ(MovedTrack%vec%w, dc, dc1)

       aTrack%p%fm%p(4) = e1
       call csetDirCos(dc1, aTrack)
       call ce2p(aTrack)
!           Nproduced = Nproduced + 1 
       idx = Nproduced + 2
!           Pwork(Nproduced) = aTrack.p
       Pwork(idx) = aTrack%p
!            knock on electron
       tmp=1.d0-cosr*cosr

       if(tmp .lt. 0.d0) then
          tmp=0.d0
          cosr=1.d0
       endif
       sinr=sqrt(tmp)
       dc%r(1) = -cs*sinr
       dc%r(2) = -sn*sinr
       dc%r(3) = cosr
       call ctransVectZ(MovedTrack%vec%w, dc, dcr)
       aTrack%p%fm%p(4) = er
       call csetDirCos(dcr, aTrack)
!
       call cmkptc(kelec,  0,  -1,  aTrack%p)

       call ce2p(aTrack)
!            Nproduced = Nproduced + 1
       idx =  Nproduced + 1
       Pwork(idx) = aTrack%p

       Nproduced = Nproduced + 2
      end

==> Cosmos8.038.tar.gz: Cosmos8.038/Tracking/cinteNEP.f <==
!            treat interaction of MovedTrack for non e,g
!
      subroutine cinteNEP
      implicit none

#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
!
!
!          used to judge if user hook should be called 
!          after MovedTrack interacted.
!
!
      character*80 msg
!
!   **  ptcl stacking is done in each subroutine; should be changed
!           (except for hadronic interactions)
!////////////
!      logical deb
!      common /cccdeb/deb
!/////////////

      
!/////////
!      if(deb) then
!         write(*,*) 'NEP:proc=',  IntInfArray(ProcessNo).process
!      endif
!////////////

      if(IntInfArray(ProcessNo)%process .eq. 'knock') then
         call csetKnock
      else
         if(MovedTrack%p%code .eq. kpion) then
            call cintePion
         elseif(MovedTrack%p%code .eq. kkaon) then
            call cinteKaon
         elseif(MovedTrack%p%code .eq. knuc) then
            call cinteNuc
         elseif(MovedTrack%p%code .eq. kmuon) then
            call cinteMuon
         elseif( MovedTrack%p%code .eq. kgnuc ) then
            call cinteHeavy
         elseif(MovedTrack%p%code .ge. kalfa .and.
     *           MovedTrack%p%code .le. khvymax  ) then
!cc            elseif(MovedTrack.p.code .ge. kdeut .and.
            call cinteHeavy
         elseif(MovedTrack%p%code .eq. ktriton) then
            call cinteHeavy
         elseif(MovedTrack%p%code .eq. kdmes ) then
            call cinteDmes
         elseif(MovedTrack%p%code .eq. knnb) then
            call cintennb
         elseif(MovedTrack%p%code .eq. kddb) then
            call cinteddb
         elseif(MovedTrack%p%code .eq. keta) then
            call cinteEta
         elseif(MovedTrack%p%code .eq. kgzai) then
            call cinteGzai
         elseif(MovedTrack%p%code .eq. kbomega) then
            call cinteBomega
         elseif(MovedTrack%p%code .eq. klambda) then
            call cinteLambda
         elseif(MovedTrack%p%code .eq. klambdac) then
            call cinteLambdac
         elseif(MovedTrack%p%code .eq. ksigma) then
            call cinteSigma
         elseif(MovedTrack%p%code .eq. krho) then
            call cinterho
         elseif(MovedTrack%p%code .eq. komega) then
            call cinteomega
         elseif(MovedTrack%p%code .eq. kphi) then
            call cintephi
         elseif(MovedTrack%p%code .eq. kds ) then
            call cinteds
         elseif(MovedTrack%p%code .eq. ketap ) then
            call cinteEtap
         elseif(MovedTrack%p%code .eq. kDelta ) then
            call cinteDelta
         elseif(MovedTrack%p%code .eq. ktau ) then
            call cinteTau
         else
            write(0,*) ' E=', MovedTrack%p%fm%p(4)
            write(msg, *) ' ptcl =', MovedTrack%p%code,
     *        ' interaction=', 
     *         IntInfArray(ProcessNo)%process,
     *        ' should not occure'
            call cerrorMsg(msg, 0)
         endif
      endif
      end     
      subroutine csetKnock
      implicit none
#include  "Zcode.h"
#include  "Ztrack.h"
#include  "Ztrackv.h"
!

       real*8 e1, er, tmp, cos1, cosr,  cs, sn, sinr, sine
       type(coord)::dc
       type(coord)::dc1
       type(coord)::dcr
!
       type(track)::aTrack

       integer idx   ! 
 
       aTrack = MovedTrack
!                                surv  elec  surv   elec
       call  cKnockea(aTrack%p,  e1,   er,  cos1,   cosr)
!        we must put recoil electron into Pwork first and
!        then survival high energy ptcl.  This order will
!        be reversed when particles in Pwork are stacked.
!      (this inversion is to manage nuclear interactions where
!       the leading particle is put last in Pwork).
!        Therefore the low energy electron is first extracted.
!  Otherwise, stack overflow may take place for inclinde muons
!     (<= v5.33)
!         idx is to resolve the problem with a minumum effort.
!
!          survival paticle angle negligible always so
!          you may  put dc.r = (0, 0, 1)
       tmp=1.d0-cos1*cos1
       if(tmp .lt. 0.d0) then
          tmp=0.d0
          cos1=1.d0
       endif
       sine=sqrt(tmp)

       call kcossn(cs, sn)
       dc%r(1) = cs*sine
       dc%r(2) = sn*sine
       dc%r(3) = cos1
!           convert angle
       call ctransVectZ(MovedTrack%vec%w, dc, dc1)

       aTrack%p%fm%p(4) = e1
       call csetDirCos(dc1, aTrack)
       call ce2p(aTrack)
!           Nproduced = Nproduced + 1 
       idx = Nproduced + 2
!           Pwork(Nproduced) = aTrack.p
       Pwork(idx) = aTrack%p
!            knock on electron
       tmp=1.d0-cosr*cosr

       if(tmp .lt. 0.d0) then
          tmp=0.d0
          cosr=1.d0
       endif
       sinr=sqrt(tmp)
       dc%r(1) = -cs*sinr
       dc%r(2) = -sn*sinr
       dc%r(3) = cosr
       call ctransVectZ(MovedTrack%vec%w, dc, dcr)
       aTrack%p%fm%p(4) = er
       call csetDirCos(dcr, aTrack)
!
       call cmkptc(kelec,  0,  -1,  aTrack%p)

       call ce2p(aTrack)
!            Nproduced = Nproduced + 1
       idx =  Nproduced + 1
       Pwork(idx) = aTrack%p

       Nproduced = Nproduced + 2
      end
